unit rmKnob;

interface

uses
   SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
   Forms, Dialogs, math,comctrls;

type
  TKnobChangeEvent = procedure(Sender : TObject; Value : Longint) of object;
  TMarkStyle = (msLine, msCircle);

TrmKnob = class(TGraphicControl) // tCustomControl
private
  { Private declarations }
  FMaxValue: Longint;
  FMinValue: Longint;
  FPosition: Longint;
  FActiveColor: TColor;
  FInactiveColor: TColor;
  FShadow: Boolean;
  FTickLength: integer;
  FTickWidth: integer;
  FTickActiveColor: TColor;
  FTickInactiveColor: TColor;
  FDrag: Boolean;
  FKnobChangeEvent: TKnobChangeEvent;
  FollowMouse: Boolean;
  FMarkStyle: TMarkStyle;
  procedure SetMaxValue(Value: Longint);
  procedure SetMinValue(Value: Longint);
  procedure SetPosition(Value: Longint);
  procedure SetActiveColor(Value: TColor);
  procedure SetInactiveColor(Value: TColor);
  procedure SetTickLength(Value: Integer);
  procedure SetShadow(Value: boolean);
  procedure SetTickWidth(Value: Integer);
  procedure SetTickActiveColor(Value: TColor);
  procedure SetTickInactiveColor(Value: TColor);
  procedure SetMarkStyle(Value: TMarkStyle);
  procedure SetDrag(Value: Boolean);
  procedure UpdatePosition(X, Y: Integer);
  function  ConstrainValue(Minimum, Maximum, Value: Longint): Longint;

protected
 { Protected declarations }
  procedure KnobChange;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  procedure Paint; override;

public
  { Public declarations }
  constructor Create(AOwner: TComponent); override;

published
  { Published declarations }
  property AllowUserDrag: Boolean read FDrag write SetDrag default True;
  property ActiveColor: TColor read fActiveColor write SetActiveColor;
  property InactiveColor: TColor read fInactiveColor write SetInactiveColor;
  property Shadow: boolean read FShadow write SetShadow;
  property TickLength: integer read FTickLength write SetTickLength;
  property TickWidth: integer read FTickWidth write SetTickWidth;
  property TickActiveColor: TColor read fTickActiveColor write SetTickActiveColor;
  property TickInactiveColor: TColor read FTickInactiveColor write SetTickInactiveColor;
  property Position: Longint read FPosition write SetPosition;
  property MarkStyle: TMarkStyle read fMarkStyle write SetMarkStyle;
  property Max: Longint read FMaxValue write SetMaxValue;
  property Min: Longint read FMinValue write SetMinValue;
  property OnChange: TKnobChangeEvent read fKnobChangeEvent write fKnobChangeEvent;
  property Align;
  property Enabled;
  // property Enabled: boolean read FEnabled write SetEnabled;
  property ParentShowHint;
  property ShowHint;
  property Visible;
  property ParentColor;

end;

procedure Register;

implementation

constructor TrmKnob.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  Width := 49;
  Height := 49;
  FMaxValue := 100;
  FMinValue := 0;
  FPosition := 0;
  FMarkStyle := msCircle;
  FShadow := true;
  FTickLength := 5;
  FTickWidth := 2;
  FTickInactiveColor := ClSilver;
  FTickActiveColor := ClAqua;
  FollowMouse := false;
  FActiveColor := clMaroon;
  FInactiveColor := clGray;
  SetDrag(True);
end;

function TrmKnob.ConstrainValue(Minimum, Maximum, Value : Longint) : Longint;
begin
  Result := Value;
  if Value < Minimum then
    Result := Minimum;
  if Value > Maximum then
    Result := Maximum;
end;

procedure TrmKnob.SetMaxValue(Value : Longint);
begin
  if Value <> fMaxValue then
  begin
    FMaxValue := Value;
    if FPosition > FMaxValue then
      FPosition := FMaxValue;
    Refresh;
  end;
end;

procedure TrmKnob.SetMinValue(Value : Longint);
begin
  if Value <> fMinValue then
  begin
    FMinValue := Value;
    if FPosition < FMinValue then
      FPosition := FMinValue;
    Refresh;
  end;
end;

procedure TrmKnob.SetMarkStyle(Value : TMarkStyle);
begin
  if Value <> fMarkStyle then
  begin
    fMarkStyle := Value;
    Refresh;
  end;
end;

procedure TrmKnob.SetPosition(Value : Longint);
begin
  if Value <> FPosition then
  begin
    FPosition := ConstrainValue(fMinValue, fMaxValue, Value);
    Refresh;
    KnobChange;
  end;
end;

procedure TrmKnob.SetShadow(Value : boolean);
begin
  if Value <> FShadow then
  begin
    FShadow := Value;
    Refresh;
  end;
end;

procedure TrmKnob.SetTickLength(Value : integer);
begin
  if Value <> FTickLength then
  begin
    FTickLength := Value;
    Refresh;
  end;
end;

procedure TrmKnob.SetTickWidth(Value : integer);
begin
  if Value <> FTickWidth then
  begin
    FTickWidth := Value;
    Refresh;
  end;
end;

procedure TrmKnob.SetTickActiveColor(Value : TColor);
begin
  if Value <> fTickActiveColor then
  begin
    fTickActiveColor := Value;
    Refresh;
  end;
end;

procedure TrmKnob.SetTickInactiveColor(Value : TColor);
begin
  if Value <> fTickInactiveColor then
  begin
    fTickInactiveColor := Value;
    Refresh;
  end;
end;

procedure TrmKnob.SetActiveColor(value : TColor);
begin
  if FActiveColor <> value then
  begin
    FActiveColor := value;
    Refresh;
  end;
end;

procedure TrmKnob.SetInactiveColor(value : TColor);
begin
  if FInactiveColor <> value then
  begin
    FInactiveColor := value;
    Refresh;
  end;
end;

procedure TrmKnob.SetDrag(Value : Boolean);
begin
  if Value <> FDrag then begin
    FDrag := Value;
    Refresh;
   end;
end;

procedure TrmKnob.KnobChange;
begin
  if Assigned(fKnobChangeEvent) then
    fKnobChangeEvent(Self, FPosition);
end;

procedure TrmKnob.Paint;
var
 R: TRect;
 Bm: TBitMap;
 Co, Si, Angle: Double;
 X, Y, W, H: Integer;
 dx, dy, gx, gy: Integer;
 OuterPoint: TPoint;
begin
  Bm := TBitmap.Create;
  if (csdesigning in componentstate) then
    if Height < Width then
      Height := Width
    else
      Width := Height;

  Bm.Width := Width;
  Bm.Height := Height;
  Bm.Canvas.Brush.Color := clBtnFace;
  R.Left := 0;
  R.Top := 0;
  R.Right := Width;//- 1;
  R.Bottom := Height;// - 1;
  W := R.Right - R.Left - 4;
  H := R.Bottom - R.Top - 4;

  Bm.Canvas.FillRect(R);

  with Bm.Canvas do
  begin

    if enabled then
      Brush.Color := ActiveColor
    else
      Brush.Color := InactiveColor;
    Pen.Width := 2;
    Pen.Color := Cl3dLight;
    ellipse(1, 1, W - 1, H - 1);
    Pen.Color := ClBtnShadow;
    if FShadow then
      ellipse(3, 3, W + 2, H + 2);
      //ellipse(2, 2, W + 1, H + 1);

    Pen.Color := ClBtnface;
    Pen.Width := 1;
    RoundRect(2, 2, w, h, w, h);
    Pen.Width := 3;
    Pen.Color := TickActiveColor;

    if Position >= 0 then
    begin
      if enabled then
        Brush.Color := ActiveColor
      else
        Brush.Color := InactiveColor;
      X := W div 2;
      Y := H div 2;
      dX := W div 6;
      dY := H div 6;
      gX := W div 32;
      gY := H div 32;

      Angle := (Position - (Min + Max) / 2 ) / (Max - Min) * 5 ;

      Si := Sin(Angle);
      Co := Cos(Angle);
      OuterPoint.X := Round(X + Si * (X - dx));
      OuterPoint.Y := Round(Y - Co * (Y - dy));
      MoveTo(OuterPoint.X, OuterPoint.y);

      case MarkStyle of

        // draw a line
        msLine :
        begin
          if Enabled then
            pen.Color := FTickActiveColor
          else
            pen.Color := FTickInactiveColor;
          pen.width := FTickWidth;
          //LineTo(Round(X + Si * (X - gx)), Round(Y - Co * (Y - gy)));
          LineTo(Round(X + Si * (X - gx)), Round(Y - Co * (Y - gy)));
        end;

        // draw a circle
        msCircle :
        begin
          pen.width := 1;
          if Enabled then
          begin
            pen.Color := FTickActiveColor;
            Brush.Color := FTickActiveColor;
          end
          else begin
            Brush.Color := FTickInactiveColor;
            pen.Color := FTickInactiveColor;
          end;
          ellipse(
            OuterPoint.X - (FTickLength div 2),
            OuterPoint.Y - (FTickLength div 2),
            OuterPoint.X + (FTickLength div 2),
            OuterPoint.Y + (FTickLength div 2));
          {
          ellipse(
            OuterPoint.X - trunc(width / 15),
            OuterPoint.Y - trunc(width / 15),
            OuterPoint.X + trunc(width / 15),
            OuterPoint.Y + trunc(width / 15));
          }
          {
          RoundRect(OuterPoint.X - 3, OuterPoint.Y - 3,
                    OuterPoint.X + 3, OuterPoint.Y + 3,
                    OuterPoint.X + 3, OuterPoint.Y + 3);
          }
        end;
      end;
    end;
  end;

  Canvas.CopyMode := cmSrcCopy;
  Canvas.Draw(0, 0, Bm);
  bm.Destroy;
end;

procedure TrmKnob.UpdatePosition(X, Y: Integer);
var
  CX, CY: integer;
  R: double;
  Angle: double;
begin
  CX := Width div 2;
  CY := Height div 2;
  R := Round(sqrt(sqr(CX - X) + sqr(CY - Y))) ;
  if R = 0 then
    R := 0.0001;

  if Y < CY then
    Angle := ArcSin((X - CX) / R)
  else begin
    Angle := arcsin((CX - X) / R);
    if X > CX then
      Angle := Angle + Pi
    else
      Angle := Angle - Pi;
  end;
  Position := Round(Angle * (Max - Min) / 5 + (Min + Max) / 2);
  Refresh;
end;

procedure TrmKnob.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if FDrag = True then
  begin
    FollowMouse := True;
    UpdatePosition(X, Y);
    Refresh;
  end;
end;

procedure TrmKnob.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  FollowMouse := False;
end;

procedure TrmKnob.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove(Shift, X, Y);
  if FollowMouse = True then
  begin
    UpdatePosition(X,Y)
  end;
end;

procedure Register;
begin
  RegisterComponents('OptoElec', [TrmKnob]);
end;

end.
